from __future__ import absolute_import, unicode_literals
import os, sys

try:
    from pathlib import Path
except ImportError:
    Path = lambda x: x

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

from email.utils import parseaddr

if sys.version_info[0] == 2:
    bytes_type = str
    str_type   = unicode
else:
    bytes_type = bytes
    str_type   = str

__all__ = ['Environment']

def _parse_bool(value):
    if isinstance(value, str_type):
        value = value.lower()

    if value in (u'yes', u'true', u'True'):
        return True
    elif value in (u'no', u'false', u'False'):
        return False

    return bool(int(value))

def _parse_list(value):
    if isinstance(value, list):
        return value

    return [x.strip() for x in str_type(value).split(u',')]

def _parse_emails(value):
    value = _parse_list(value)
    return [parseaddr(x) for x in value]

_CONVERT_MAP = {
    list: _parse_list,
    set: lambda v: set(_parse_list(v)),
    tuple: lambda v: tuple(_parse_list(v)),
    bool: _parse_bool,
}

class Environment(object):
    def __init__(self, environ = os.environ, encoding = 'utf-8', errors = 'strict'):
        self.encoding = encoding
        self.errors = errors
        self.environ = environ

    def _decode(self, value):
        return value.decode(self.encoding, self.errors) if isinstance(value, bytes_type) else value

    def get(self, key, default):
        try:
            value = self.environ[key]
        except KeyError:
            return default() if callable(default) else default
        else:
            return self._decode(value)

    def get_typed(self, convert, key, default):
        # grab _parse_X if supported, else use the function directly
        convert = _CONVERT_MAP.get(convert, convert)
        return convert(self.get(key, default))

    def get_int(self, key, default):
        return self.get_typed(int, key, default)

    def get_float(self, key, default):
        return self.get_typed(float, key, default)

    def get_bool(self, key, default):
        return self.get_typed(bool, key, default)

    def get_list(self, key, default):
        return self.get_typed(list, key, default)

    def get_tuple(self, key, default):
        return self.get_typed(tuple, key, default)

    def get_set(self, key, default):
        return self.get_typed(set, key, default)

    def get_url(self, key, default):
        return self.get_typed(urlparse, key, default)

    def get_path(self, key, default):
        return self.get_typed(Path, key, default)

    def get_email(self, key, default):
        return self.get_typed(parseaddr, key, default)

    def get_email_list(self, key, default):
        return self.get_typed(_parse_emails, key, default)
